issuesQuery = """
        query Issues($next: String, $replicationKeyValue: DateTime) {
						issues(
							first: 100
							after: $next
							filter: { updatedAt: {gt: $replicationKeyValue } }
						) {
							pageInfo {
								hasNextPage
								endCursor
							}
							nodes {
								id
								title
								url
								updatedAt
								creator {
									id
									name
									email
								}
								assignee {
									id
									name
									email
								}
								project {
									id
									name
								}
								team {
									id
									name
								}
							}
						}
					}
        """



"""Stream type classes for tap-linear."""
from tap_linear.client import LinearStream
from tap_linear.schemas.issues import issuesSchema
from tap_linear.queries.issues import issuesQuery


class IssuesStream(LinearStream):

    """Define custom stream."""

    name = "Issues"
    schema = issuesSchema
    primary_keys = ["id"]
    replication_key = "updatedAt"
    query = issuesQuery



from singer_sdk import typing as th

issuesSchema = th.PropertiesList(
    th.Property("id", th.StringType),
    th.Property("title", th.StringType),
    th.Property("url", th.StringType),
    th.Property("updatedAt", th.DateTimeType),
    th.Property(
        "creator",
        th.ObjectType(
            th.Property("id", th.StringType),
            th.Property("name", th.StringType),
            th.Property("email", th.StringType),
        ),
    ),
    th.Property(
        "assignee",
        th.ObjectType(
            th.Property("id", th.StringType),
            th.Property("name", th.StringType),
            th.Property("email", th.StringType),
        ),
    ),
    th.Property(
        "project",
        th.ObjectType(
            th.Property("id", th.StringType),
            th.Property("name", th.StringType),
        ),
    ),
    th.Property(
        "team",
        th.ObjectType(
            th.Property("id", th.StringType),
            th.Property("name", th.StringType),
        ),
    ),
).to_dict()



"""GraphQL client handling, including LinearStream base class."""

from datetime import timedelta
import requests
from typing import Any, Optional, Iterable
from singer_sdk.authenticators import APIKeyAuthenticator
from singer_sdk.streams import GraphQLStream


class LinearStream(GraphQLStream):
    """Linear stream class."""

    @property
    def authenticator(self) -> APIKeyAuthenticator:
        return APIKeyAuthenticator.create_for_stream(
            self,
            key="Authorization",
            value=self.config.get("auth_token"),
            location="header",
        )

    @property
    def url_base(self) -> str:
        return self.config.get("api_url")

    @property
    def http_headers(self) -> dict:
        """Return the http headers needed."""
        headers = {}
        if "user_agent" in self.config:
            headers["User-Agent"] = self.config.get("user_agent")
        return headers

    def get_next_page_token(
        self, response: requests.models.Response, previous_token: Optional[Any]
    ) -> Any:
        """Return the next page token."""
        resp_json = response.json()
        if resp_json["data"]["issues"]["pageInfo"]["hasNextPage"]:
            return resp_json["data"]["issues"]["pageInfo"]["endCursor"]
        else:
            return None

    def prepare_request_payload(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Optional[dict]:
        """Return the request payload."""
        value = (self.get_starting_timestamp(context) + timedelta(seconds=1)).strftime(
            "%Y-%m-%dT%H:%M:%SZ"
        )
        self.logger.info(f"Previous state timestamp: {value}")
        body = {
            "query": self.query,
            "variables": {"next": next_page_token, "replicationKeyValue": value},
        }
        return body

    def parse_response(self, response: requests.Response) -> Iterable[dict]:
        """Parse the response and return an iterator of result rows."""
        resp_json = response.json()
        for row in resp_json["data"]["issues"]["nodes"]:
            yield row



"""Tests standard tap features using the built-in SDK tests library."""

import datetime
import os

from singer_sdk.testing import get_standard_tap_tests

from tap_linear.tap import TapLinear

SAMPLE_CONFIG = {
    "start_date": datetime.datetime.now(datetime.timezone.utc).strftime(
        "%Y-%m-%dT%H:%M:%SZ"
    ),
    "auth_token": os.environ.get("LINEAR_AUTH_TOKEN"),
}


def test_standard_tap_tests():
    """Run standard tap tests from the SDK."""
    tests = get_standard_tap_tests(TapLinear, config=SAMPLE_CONFIG)
    for test in tests:
        test()



"""Linear tap class."""

from typing import List
from singer_sdk import Tap, Stream
from singer_sdk import typing as th
from tap_linear.streams import IssuesStream

STREAM_TYPES = [
    IssuesStream,
]


class TapLinear(Tap):
    """Linear tap class."""

    name = "tap-linear"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "auth_token",
            th.StringType,
            required=True,
            description="The token to authenticate against the API service",
        ),
        th.Property(
            "api_url",
            th.StringType,
            default="https://api.linear.app/graphql",
            description="The URL for the API service",
        ),
        th.Property(
            "start_date",
            th.DateTimeType,
            description="The earliest record date to sync",
            default="2010-01-01T00:00:00.000Z",
        ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        return [stream_class(tap=self) for stream_class in STREAM_TYPES]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_linear', 'tap_linear.queries', 'tap_linear.schemas', 'tap_linear.tests']

package_data = \
{'': ['*']}

install_requires = \
['black>=21.9b0,<22.0', 'requests>=2.25.1,<3.0.0', 'singer-sdk>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['tap-linear = tap_linear.tap:TapLinear.cli']}

setup_kwargs = {
    'name': 'tap-linear',
    'version': '0.1.1',
    'description': '`tap-linear` is a Singer tap for Linear, built with the Meltano SDK for Singer Taps.',
    'long_description': None,
    'author': 'Harisaran G',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<3.11',
}


setup(**setup_kwargs)
